@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2019
+    written by Marc Ihm 1995-2020
     more info at www.yabasic.de
 
     flow.c --- code for subroutines and flow-control
@@ -78,12 +78,13 @@ check_return_value (struct command *cmd)	/* check return value of function */
         s = stackhead->prev;
         if (s->type == stNUMBER) {
             sprintf (string, "subroutine returns number %g", s->value);
-        } else if (s->type == stSTRING)
+        } else if (s->type == stSTRING) {
             sprintf (string, "subroutine returns string '%s'",
                      (char *) s->pointer);
-        else
+        } else {
             sprintf (string, "subroutine returns something strange (%d)",
                      s->type);
+	}
         error (sDEBUG, string);
     }
 }
@@ -673,18 +674,32 @@ skiponce (struct command *cmd)	/* skip next command exectly once */
     if (cmd->tag) {
         current = current->next;
     }
+    if (severity_threshold <= sDEBUG) {
+	if (cmd->tag) {
+	    error (sDEBUG, "skipping next command");
+	} else {
+	    error (sDEBUG, "not Skipping next command");
+	}
+    }
     cmd->tag = 0;
 }
 
 
 void
-resetskiponce (struct command *cmd)	/* find and reset next skip */
+resetskiponce (struct command *cmd, int n)	/* find and reset nth skip */
 {
     struct command *c;
+    int i;
 
     c = cmd;
-    while (c->type != cSKIPONCE) {
-        c = c->next;
+
+    for(i=0;i<n;i++) {
+	while (c->type != cSKIPONCE) {
+	    c = c->next;
+	}
+	if (i == 0 && n == 2) {
+	    c = c->next;
+	}
     }
     c->tag = 1;
 }
@@ -897,9 +912,7 @@ startfor (void)			/* compute initial value of for-variable */
     struct stackentry *p;
 
     p = push ();
-    p->value =
-        stackhead->prev->prev->prev->prev->value -
-        stackhead->prev->prev->value;
+    p->value = stackhead->prev->prev->prev->prev->value;
     p->type = stNUMBER;
 
     return;
