@@ -1,7 +1,7 @@
 /*
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc Ihm 1995-2019
+    written by Marc Ihm 1995-2020
     more info at www.yabasic.de
 
     main.c --- main() and auxilliary functions
@@ -23,7 +23,7 @@
 /* ------------- defines ---------------- */
 
 #define DONE {current=current->next;break;}	/* reduces type-work */
-#define COPYRIGHT "Copyright 1995-2019 by Marc Ihm, according to the MIT License"
+#define COPYRIGHT "Copyright 1995-2020 by Marc Ihm, according to the MIT License"
 /* NOTE: Including whatever timestamp into the banner would break the reproducible build */
 #define BANNER \
 "\nThis is yabasic version " VERSION ",\nconfigured for "\
@@ -1184,6 +1184,7 @@ initialize (void)
     cexplanation[cSKIPPER] = "SKIPPER";
     cexplanation[cSKIPONCE] = "SKIPONCE";
     cexplanation[cRESETSKIPONCE] = "RESETSKIPONCE";
+    cexplanation[cRESETSKIPONCE2] = "RESETSKIPONCE2";
     cexplanation[cNOP] = "NOP";
     cexplanation[cEND_FUNCTION] = "END_FUNCTION";
     cexplanation[cDIM] = "DIM";
@@ -1567,7 +1568,10 @@ run_it ()
                 skiponce (current);
                 DONE;
             case cRESETSKIPONCE:
-                resetskiponce (current);
+                resetskiponce (current,1);
+                DONE;
+            case cRESETSKIPONCE2:
+                resetskiponce (current,2);
                 DONE;
             case cBREAK_MULTI:
                 mybreak (current);
